{---------------------------------------------------------------------------
 | Copyright (C) 2004 Dallas Semiconductor MAXIM Corporation, All Rights Reserved.
 |
 | Permission is hereby granted, free of charge, to any person obtaining a
 | copy of this software and associated documentation files (the "Software"),
 | to deal in the Software without restriction, including without limitation
 | the rights to use, copy, modify, merge, publish, distribute, sublicense,
 | and/or sell copies of the Software, and to permit persons to whom the
 | Software is furnished to do so, subject to the following conditions:
 |
 | The above copyright notice and this permission notice shall be included
 | in all copies or substantial portions of the Software.
 |
 | THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 | OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 | MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 | IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 | OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 | ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 | OTHER DEALINGS IN THE SOFTWARE.
 |
 | Except as contained in this notice, the name of Dallas Semiconductor MAXIM
 | shall not be used except as stated in the Dallas Semiconductor MAXIM
 | Branding Policy.
 |---------------------------------------------------------------------------

 Utility to be used during the Windows 32-bit 1-Wire driver install.  This
 utility takes on 4 different personalities depending on the command line
 parameter.  If 'three' command line parameters are provided it will display
 debug information (for example: chkusr U Z debug)

 Modes:
  chkusr U - check user for pre-install warnings
  chkusr J - checks for the existence of Microsoft's JVM
  chkusr K - install registry keys on some platforms for iButton TMEX
  chkusr W - provide post-install warnings
}

unit chkuser;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  TChkUserMain = class(TForm)
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    doDebug : Boolean;
    Function WriteRegStr(BaseName,KeyName,ValueName,StringValue: String): Boolean;
    Function WriteRegLong(BaseName,KeyName,ValueName: String; LongValue: LongInt): Boolean;
    Function GetSystemDir: String;
    Function GetWindowsDir: String;
    Function GetCurrentDir: String;
  end;

var
  ChkUserMain: TChkUserMain;

implementation

uses SetupAPI;

{$R *.DFM}

{----------------------------------------------------------------------------
 | Form create event.
 |
 | Parameter:
 |  Sender - sender object
}
procedure TChkUserMain.FormCreate(Sender: TObject);
Var
   user : String;
   RtSize : DWORD;
   RetBuf : Array[0..255] of Char;
   blNTFlag,pass, JVMPresence : Boolean;
   dwVersion,dwWindowsMajorVersion,dwWindowsMinorVersion,dwBuild : DWORD;
   FileNameSystem, FileNameWindows:  string;
   InstallPath : String;
   InstallPathPChar : Array[0..128] of Char;
   MessageString : Array[0..512] of Char;
begin
   JVMPresence := false;
   {Check for no command line }
   If ParamCount = 0 Then
      Application.Terminate;

   InstallPath := '';
   messageString := '';

   {Check for debug mode }
   doDebug := (ParamCount > 2);

   { attempt to get the user name }
   RtSize := Sizeof(RetBuf);
   If (GetUserName(@RetBuf[0],RtSize)) Then
       user := StrPas(RetBuf)
   Else
       user := 'nothing';

   // Get major and minor version numbers of Windows
   dwVersion := GetVersion();

   // extract NT flag
   blNTFlag := (dwVersion < $80000000);

   // get versions
   dwWindowsMajorVersion :=  dwVersion And $FF;
   dwWindowsMinorVersion :=  (dwVersion shr 8) And $FF;

   // get build number
   if (blNTFlag) Then
      dwBuild := ((dwVersion shr 16) And $FFFF)
   else if (dwWindowsMajorVersion < 4) Then     // Win32s
      dwBuild := ((dwVersion shr 16) And $7FFF)
   else         // Windows 95 -- No build numbers provided
      dwBuild :=  0;

   // DEBUG
   If doDebug then
      ShowMessage('User=' + user + ' NT=' + IntToStr(Integer(blNTFlag)) +
               ' Major=' + IntToStr(dwWindowsMajorVersion) + ' Minor=' +
               IntToStr(dwWindowsMinorVersion) + ' Build=' + IntToStr(dwBuild));

   { User check }
   If (AnsiSameStr('U', ParamStr(1))) Then
   Begin
      // Non-administrator warning in NT
      If ((CompareText(user, 'Administrator') <> 0) And blNTFlag) Then
         Application.MessageBox('WARNING, you must be logged in as Administrator to install this package!' +
                   Chr($0D) +  Chr($0D) + 'If you are not Administrator, stop this installation NOW and re-login.',
                   'Check User', mb_OK + mb_IconWarning + mb_ApplModal + mb_TopMost);

      // No USB warning
      If ((dwWindowsMajorVersion = 4) And (dwWindowsMinorVersion < 10)) Or
          (dwWindowsMajorVersion < 4) Then
         Application.MessageBox('WARNING, the 1-Wire USB drivers will be unavailable on this Operating System!' +
                   Chr($0D) +  Chr($0D) + '(minimum OS: Windows 2000, Windows 98)',
                   'Check OS', mb_OK + mb_IconWarning + mb_ApplModal + mb_TopMost);

      // If in Win98/Win2K Unplug USB warning
      If (((Not blNTFlag) And (dwWindowsMajorVersion = 4) And (dwWindowsMinorVersion >= 10)) Or
         (blNTFlag And (dwWindowsMajorVersion = 5))) Then
         Application.MessageBox('WARNING, if you currently have a 1-Wire USB device plugged in, ' +
                   Chr($0D) +  'you must UNPLUG it for the driver to be updated.',
                   'USB', mb_OK + mb_IconInformation + mb_ApplModal + mb_TopMost);

   End;

   { No Microsoft JVM Warning }
   If (AnsiSameStr('J', ParamStr(1))) Then
   Begin
      // Microsoft's JVM not installed warning (search for presence of JView.exe.)
      // retrieve environment variable for the Windows\System path...
      FileNameSystem := GetSystemDir + '\JView.exe';
      FileNameWindows := GetWindowsDir + '\JView.exe';
      // check for jview.exe in Windows and/or System directory.
      if FileExists(FileNameSystem) then
      begin
         JVMPresence := True;
      end;
      if FileExists(FileNameWindows) then
      begin
         JVMPresence := True;
      end;
      if (not JVMPresence) then
         Application.MessageBox('WARNING, the Microsoft Java Virtual Machine (JVM) has not been detected on this machine.  '
                   + Chr($0D) + 'The 1-Wire COM object will be unavailable until it is installed.' +
                   Chr($0D) +  Chr($0D) + 'Please see http://www.microsoft.com to download the latest JVM from Microsoft.',
                   'Microsoft JVM Not Found', mb_OK + mb_IconWarning + mb_ApplModal + mb_TopMost);
   End;

   { Registry KEY install with K switch }
   If (AnsiSameStr('K', ParamStr(1))) Then
   Begin
      pass := True;

      // determine the Install Path -- the default install directory
      // this is passed as the second command line parameter in
      // InstallShield Express.

      InstallPath := ParamStr(2);
      StrPCopy(@InstallPathPChar[0],InstallPath);

      // In NT, so install Parallel Keys
      If (blNTFlag) Then
      Begin
         // DEBUG
         If doDebug then
            ShowMessage('in WinNT so install Parallel keys');

         pass := True;

         If (Not WriteRegStr('System\CurrentControlSet\Services',
                     'DS1410D',
                     'Group',
                     'Extended Base')) Then
            pass := False;

         If (Not WriteRegLong('System\CurrentControlSet\Services',
                     'DS1410D',
                     'ErrorControl',
                     $00000001)) Then
            pass := False;

         If (Not WriteRegLong('System\CurrentControlSet\Services',
                     'DS1410D',
                     'Start',
                     $00000002)) Then
            pass := False;

         If (Not WriteRegLong('System\CurrentControlSet\Services',
                     'DS1410D',
                     'Tag',
                     $00000002)) Then
            pass := False;

         If (Not WriteRegLong('System\CurrentControlSet\Services',
                     'DS1410D',
                     'Type',
                     $00000001)) Then
            pass := False;

         If (Not pass) Then
            Application.MessageBox('WARNING, could not set the registry keys for Parallel (DS1410E) support!',
                   'Set Parallel Registry Keys', mb_OK + mb_IconWarning + mb_ApplModal + mb_TopMost);
      End;
      MessageString := Chr($0D) + '1) If you have a 1-Wire USB adapter (DS9490), plug it in now.' + Chr($0D) +
            '2) If this is not a first-time installation, click OK to continue, else goto next step.' + Chr($0D) +
            '3) Wait for Windows to detect your new hardware.  If you have trouble, browse with Hardware Wizard to:' +  Chr($0D) +  '     ';
      StrCat(@MessageString[0], @InstallPathPChar[0]);
      If (pass) Then
      Begin
          Application.MessageBox(MessageString,'USB Instructions', mb_OK + mb_IconInformation + mb_Applmodal + mb_TopMost);
      End
   End;

    { Pre-install of USB driver }
   If (AnsiSameStr('P', ParamStr(1))) Then
   Begin
      pass := True;

      // determine the Install Path -- the default install directory
      // this is passed as the second command line parameter in
      // MSI Installer.

      InstallPath := ParamStr(2);
      StrPCopy(@InstallPathPChar[0],InstallPath);
      StrCat(@InstallPathPChar[0], 'DS2490.inf');

      // check for USB support (Win 98 and above)
      If ((dwWindowsMajorVersion = 4) And (dwWindowsMinorVersion >= 10)) Or
          (dwWindowsMajorVersion > 4) Then
      Begin
         // Load SetupAPI
         LoadSetupApi;
         
         // Perform pre-install
         SetupCopyOEMInf(@InstallPathPChar[0], Nil, SPOST_PATH, SP_COPY_NEWER_ONLY, NIL, 256, Nil, Nil);
         
         // Unload SetupAPI
         UnloadSetupApi;
      End;
   End;

   { port-install Warnings }
   If (AnsiSameStr('W', ParamStr(1))) Then
   Begin
      // If in Win98/Win2K then warn about possible need for reboot
      If (((Not blNTFlag) And (dwWindowsMajorVersion = 4) And (dwWindowsMinorVersion >= 10)) Or
         (blNTFlag And (dwWindowsMajorVersion = 5))) Then
         Application.MessageBox(
            'NOTE:' + Chr($0D) +  Chr($0D) +
            '1)  If you are new to 1-Wire USB and have a device, plug it in now.' + Chr($0D) +
            '     If it is not recognized or you are an existing 1-Wire USB user, then RE-BOOT' +  Chr($0D) +
            '3)  If the Hardware Wizard cannot locate the USB drivers, then browse to' +  Chr($0D) +
            '     C:\Program Files\Dallas Semiconductor\Install',
            'BOOT Option', mb_OK + mb_IconInformation + mb_ApplModal + mb_TopMost);
   End;
   // end application
   Application.Terminate;
end;

{----------------------------------------------------------------------------
 | Add a registry key and string value. Will display debug message box info
 | if member boolean 'doDebug' is true.
 |
 | Parameter:
 |  BaseName - base key
 |  KeyName - key to add under base
 |  ValueName - Name of the value under key, empty if default key value
 |  StringValue - string value to set 'value' to
 |
 | Return: 'true' if the operation was a success.
}
Function TChkUserMain.WriteRegStr(BaseName,KeyName,ValueName,StringValue: String): Boolean;
Var
   BaseKey,NewKey : HKEY;
   rtval : Boolean;
   Exhist : DWORD;
   WrBuf : Array[0..255] of Char;
   TempBuf : Array[0..255] of Char;
Begin
   rtval := False;

   { open base key }
   If (RegOpenKeyEx(HKEY_LOCAL_MACHINE,StrPCopy(TempBuf,BaseName),0,
        KEY_WRITE,BaseKey) = ERROR_SUCCESS) Then
   Begin
      { create the key }
      If (RegCreateKeyEx(BaseKey,StrPCopy(TempBuf,KeyName),0,'Per-User Data',
             REG_OPTION_NON_VOLATILE,KEY_WRITE,nil,NewKey,@Exhist)
             = ERROR_SUCCESS) Then
      Begin
         { check if this new key has any values }
         If (Length(ValueName) <> 0) Then
         Begin
            { set the value name }
            StrPCopy(WrBuf,StringValue);
            { set the value under the key }
            If (RegSetValueEx(NewKey,StrPCopy(TempBuf,ValueName),0,REG_SZ,@WrBuf[0],
                Length(StringValue)) = ERROR_SUCCESS) Then
              rtval := True;
         End
         Else
            rtval := True;
         RegCloseKey(NewKey);
      End;
      RegCloseKey(BaseKey);
   End;

   // Debug
   If doDebug Then
      ShowMessage(BaseName + '\' + KeyName + Chr($0D) + Chr($0D) + ValueName +
                  '=' + StringValue + Chr($0D) +
                  Chr($0D) + IntToStr(Integer(rtval)));

   { set the return value }
   WriteRegStr := rtval;
End;





{----------------------------------------------------------------------------
 | Add a registry key and long(DWORD) value. Will display debug message box info
 | if member boolean 'doDebug' is true.
 |
 | Parameter:
 |  BaseName - base key
 |  KeyName - key to add under base
 |  ValueName - Name of the value under key, empty if default key value
 |  LongValue - long value to set 'value' to
 |
 | Return: 'true' if the operation was a success.
}
Function TChkUserMain.WriteRegLong(BaseName,KeyName,ValueName: String; LongValue: LongInt): Boolean;
Var
   BaseKey,NewKey : HKEY;
   rtval : Boolean;
   Exhist : DWORD;
   TempBuf : Array[0..255] of Char;
Begin
   rtval := False;

   { open base key }
   If (RegOpenKeyEx(HKEY_LOCAL_MACHINE,StrPCopy(TempBuf,BaseName),0,
        KEY_WRITE,BaseKey) = ERROR_SUCCESS) Then
   Begin
      { create the key }
      If (RegCreateKeyEx(BaseKey,StrPCopy(TempBuf,KeyName),0,'Per-User Data',
             REG_OPTION_NON_VOLATILE,KEY_WRITE,nil,NewKey,@Exhist)
             = ERROR_SUCCESS) Then
      Begin
         { check if this new key has any values }
         If (Length(ValueName) <> 0) Then
         Begin
            { set the value under the key }
            If (RegSetValueEx(NewKey,StrPCopy(TempBuf,ValueName),0,REG_DWORD,@LongValue,
                 SizeOf(LongValue)) = ERROR_SUCCESS) Then
               rtval := True;
         End
         Else
            rtval := True;
         RegCloseKey(NewKey);
      End;
      RegCloseKey(BaseKey);
   End;

   // Debug
   If doDebug Then
      ShowMessage(BaseName + '\' + KeyName + Chr($0D) + Chr($0D) + ValueName +
                  '=' + IntToStr(LongValue) + Chr($0D) +
                  Chr($0D) + IntToStr(Integer(rtval)));

   { set the return value }
   WriteRegLong := rtval;
End;

{----------------------------------------------------------------------------
 | Get the Windows\System(32) directory for this Microsoft OS.
 |
 | Parameter:
 |  (none)
 |
 | Return: String of the System directory (?:\...)
}
function TChkUserMain.GetSystemDir: String;
var
  SystemDir: PChar;
begin
  GetMem(SystemDir, 144);
  GetSystemDirectory(SystemDir, 144);
  Result:= StrPas(SystemDir); {Returns System Directory}
  FreeMem(SystemDir, 144);
end;

{----------------------------------------------------------------------------
 | Get the Windows directory for this Microsoft OS.
 |
 | Parameter:
 |  (none)
 |
 | Return: String of the System directory (?:\...)
}
function TChkUserMain.GetWindowsDir: String;
var
  WindowsDir: PChar;
begin
  GetMem(WindowsDir, 144);
  GetWindowsDirectory(WindowsDir, 144);
  Result:= StrPas(WindowsDir); {Returns Windows Directory}
  FreeMem(WindowsDir, 144);
end;

{----------------------------------------------------------------------------
 | Get the Current directory for this Microsoft OS.
 |
 | Parameter:
 |  (none)
 |
 | Return: String of the Current directory (?:\...)
}
function TChkUserMain.GetCurrentDir: String;
var
  CurrentDir: PChar;
begin
  GetMem(CurrentDir, 144);
  GetCurrentDirectory(144, CurrentDir);
  Result:= StrPas(CurrentDir); {Returns Current Directory}
  FreeMem(CurrentDir, 144);
end;

end.
