{--------------------------------------------------------------------------
 | Copyright (C) 1992,1993,1994,1995,1997,1998 Dallas Semiconductor Corporation.
 | All rights Reserved. Printed in U.S.A.
 | This software is protected by copyright laws of
 | the United States and of foreign countries.
 | This material may also be protected by patent laws of the United States
 | and of foreign countries.
 | This software is furnished under a license agreement and/or a
 | nondisclosure agreement and may only be used or copied in accordance
 | with the terms of those agreements.
 | The mere transfer of this software does not imply any licenses
 | of trade secrets, proprietary technology, copyrights, patents,
 | trademarks, maskwork rights, or any other form of intellectual
 | property whatsoever. Dallas Semiconductor retains all ownership rights.
 |--------------------------------------------------------------------------

   Auto Detect Form

   3.10 -> 3.11Beta - Change MicroLan to 1-Wire Net
}
unit AutoUn;

interface

uses
  WinProcs, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, WinTypes;

type
  TAutoForm = class(TForm)
    CancelButton: TButton;
    Panel1: TPanel;
    Label1: TLabel;
    AutoTimer: TTimer;
    ProgressBar: TShape;
    BackBar: TShape;
    procedure FormShow(Sender: TObject);
    procedure AutoTimerTimer(Sender: TObject);
    procedure ShowProgress;
    procedure CancelButtonClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    CheckPortNum, CheckTab : SmallInt;
    PortFound : Boolean;
    SkipLPT : Boolean;
    ProgressCount,ProgressMax : Integer;
  end;

var
  AutoForm: TAutoForm;

implementation

Uses
   PortUn;

{$R *.DFM}

{----------------------------------------------------------------------------}
{ On show event to start the auto-detect
}
procedure TAutoForm.FormShow(Sender: TObject);
begin
   { reset the search }
   CheckTab := 0;
   CheckPortNum := 0;
   PortFound := False;
   ProgressMax := PortForm.DeviceTab.Tabs.Count * 16;
   ProgressCount := 0;
   ShowProgress;
   { start the timer }
   AutoTimer.Enabled := True;
end;

{----------------------------------------------------------------------------}
{ Timer event to check a port
}
procedure TAutoForm.AutoTimerTimer(Sender: TObject);
Var
   tstr : Array[0..255] of Char;
   msg : String;
begin
   { dis-enable the timer during the check }
   AutoTimer.Enabled := False;

   { set port unit to the next port }
   PortForm.TPortNum := CheckPortNum;
   PortForm.TPortType := PortForm.PortTypeValues[CheckTab];
   { show the port }
   PortForm.SelectPort(PortForm.TPortNum,PortForm.TPortType);

   { check for LPT exception }
   If (Not (SkipLPT And (PortForm.PortTypeLabel.Caption = 'LPT'))) Then
   Begin
      { check the port }
      If (PortForm.PortCheck) Then
      Begin
         PortFound := True;
         msg := '1-Wire Net Adapter: ' + PortForm.PortName +  ' discovered.' +
            Chr($0A) + Chr($0D) + Chr($0A) + Chr($0D) +
            'Do you want this as your default 1-Wire Net port?' + Chr($0A) + Chr($0D) +
            'Click "YES" to select or "No" to continue Auto-Detect.';
         If (Application.MessageBox(StrPCopy(tstr,msg),
            '1-Wire Net Port Selection',
             mb_ApplModal Or mb_YesNo Or mb_IconInformation) = IDYES) Then     {}
         Begin
            AutoForm.ModalResult := IDOK;
            AutoForm.Hide;
         End;
      End;
   End;

   { get the next port if we are still visible }
   If (AutoForm.Visible) Then
   Begin
      { set the next port }
      If (CheckPortNum < 15) Then
      Begin
         Inc(CheckPortNum);
         { re-enable the timer }
         AutoTimer.Enabled := True;
      End
      Else
      Begin
         If (CheckTab < (PortForm.DeviceTab.Tabs.Count - 1)) Then
         Begin
            Inc(CheckTab);
            CheckPortNum := 0;
            { re-enable the timer }
            AutoTimer.Enabled := True;
         End
         { no more to check }
         Else
         Begin
            If (PortFound) Then
               msg := 'Auto-Detect has reached the end of the search without' +
                   Chr($0A) + Chr($0D) + 'finding any additional 1-Wire Net ports.' +
                   Chr($0A) + Chr($0D) + Chr($0A) + Chr($0D) +
                   'Would you like to retry Auto-Detect?'
            Else
               msg := 'Auto-Detect could not find any 1-Wire Net ports!' + Chr($0A) + Chr($0D) +
               'Check the following: ' +  Chr($0A) + Chr($0D) +
               Chr($0A) + Chr($0D) +
               '  1. 1-Wire Net adapter connected? ' + Chr($0A) + Chr($0D) +
               '  2. Is it connected to working port? ' + Chr($0A) + Chr($0D) +
               '  3. Is the 1-Wire communication channel "shorted"? ' + Chr($0A) + Chr($0D) +
               Chr($0A) + Chr($0D) + 'Retry Auto-Detect?';

            If (Application.MessageBox(StrPCopy(tstr,msg),
               '1-Wire Net Port Selection',
               MB_APPLMODAL Or MB_RETRYCANCEL Or MB_ICONEXCLAMATION) = ID_RETRY) Then
            Begin
               { reset the search }
               CheckTab := 0;
               CheckPortNum := 0;
               PortFound := False;
               ProgressMax := PortForm.DeviceTab.Tabs.Count * 16;
               ProgressCount := 0;
               ShowProgress;
               { start the timer }
               AutoTimer.Enabled := True;
            End
            Else
            Begin
               { set back to 'first' port }
               PortForm.TPortNum := 1;
               PortForm.TPortType := PortForm.PortTypeValues[0];
               { show the port }
               PortForm.SelectPort(PortForm.TPortNum,PortForm.TPortType);

               AutoForm.ModalResult := ID_CANCEL;
               AutoForm.Hide;
            End;
         End;
      End;
   End;

   Inc(ProgressCount);
   ShowProgress;
end;

{----------------------------------------------------------------------------}
{ Show the progress
}
procedure TAutoForm.ShowProgress;
Begin
   ProgressBar.Left := BackBar.Left;
   ProgressBar.Top := BackBar.Top;
   ProgressBar.Width := (ProgressCount * BackBar.Width) div ProgressMax;
End;

{----------------------------------------------------------------------------}
{ Cancel the auto-detect
}
procedure TAutoForm.CancelButtonClick(Sender: TObject);
begin
   { dis-enable the timer }
   AutoTimer.Enabled := False;
   AutoForm.ModalResult := ID_CANCEL;
   AutoForm.Hide;
   { set back to 'first' port }
   PortForm.TPortNum := 1;
   PortForm.TPortType := PortForm.PortTypeValues[0];
   { show the port }
   PortForm.SelectPort(PortForm.TPortNum,PortForm.TPortType);
end;

end.
