{--------------------------------------------------------------------------
 | Copyright (C) 1992,1993,1994,1995,1997,1998 Dallas Semiconductor Corporation.
 | All rights Reserved. Printed in U.S.A.
 | This software is protected by copyright laws of
 | the United States and of foreign countries.
 | This material may also be protected by patent laws of the United States
 | and of foreign countries.
 | This software is furnished under a license agreement and/or a
 | nondisclosure agreement and may only be used or copied in accordance
 | with the terms of those agreements.
 | The mere transfer of this software does not imply any licenses
 | of trade secrets, proprietary technology, copyrights, patents,
 | trademarks, maskwork rights, or any other form of intellectual
 | property whatsoever. Dallas Semiconductor retains all ownership rights.
 |--------------------------------------------------------------------------

   Port selection unit for SetPort.

   3.10 -> 3.11Beta - Change MicroLan to 1-Wire Net
}
unit Portun;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Grids, ExtCtrls, StdCtrls, Buttons, TMEXUnit, Spin, Tabs,
  WtGlob, AutoUn;

type
  TPortForm = class(TForm)
    CancelButton: TButton;
    NextButton: TButton;
    BackButton: TButton;
    LineBevel: TBevel;
    ImagePanel: TPanel;
    StImage: TImage;
    TMEXPanel: TPanel;
    Image1: TImage;
    CenterPanel: TPanel;
    DeviceTab: TTabSet;
    SelectBox: TGroupBox;
    PortTypeLabel: TLabel;
    Label1: TLabel;
    Label2: TLabel;
    AdapterLabel: TLabel;
    Label3: TLabel;
    PortNumSpin: TSpinEdit;
    AdvanceGroup: TGroupBox;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    VersionLabel: TLabel;
    DateLabel: TLabel;
    MessureLabel: TLabel;
    MiscMemo: TMemo;
    InfoPanel: TPanel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    QuickInfoLabel: TLabel;
    AutoDetectButton: TButton;
    procedure FormCreate(Sender: TObject);
    procedure CancelButtonClick(Sender: TObject);
    procedure NextButtonClick(Sender: TObject);
    Function  RegWriteValue(KeyBuf,ValueBuf : PChar): Boolean;
    Function  getToken(verStr : String; token : SmallInt): String;
    procedure FormShow(Sender: TObject);
    procedure DeviceTabChange(Sender: TObject; NewTab: Integer;
      var AllowChange: Boolean);
    procedure BackButtonClick(Sender: TObject);
    Procedure GetDefaults;
    procedure AutoDetectButtonClick(Sender: TObject);
  private
    SHandle : LongInt;          { TMEX Session handle }
    StateBuf : Array[0..5120] of Byte;
    EatEvent : Boolean;
    GotCancel : Boolean;
    { Private declarations }
  public
    { Public declarations }
    PanelMode : SmallInt;   { mode for multi-panel display }
    InWinNT : Boolean;
    PortName : String;
    PortTypeValues: Array[0..15] of SmallInt;
    TPortNum,TPortType: SmallInt;
    HideErrorMsgs : Boolean;
    Function  PortCheck: Boolean;
    Function  SelectPort(NPortNum, NPortType : SmallInt): Boolean;
  end;

var
  PortForm: TPortForm;

Const
   { constants for tokenization of version string }
   TOKEN_ABRV = 0;
   TOKEN_DEV  = 1;
   TOKEN_VER  = 2;
   TOKEN_DATE = 3;
   TOKEN_TAIL = 255;
   { panel modes }
   PANEL_INFO = 0;
   PANEL_PORT = 1;

implementation

{$R *.DFM}

{----------------------------------------------------------------------------}
{ Setup the known types in the port/type grid
}
procedure TPortForm.FormCreate(Sender: TObject);
Var
   x,cnt: SmallInt;
   buf : array[0..200] of Char;
   {$IFDEF WIN32}
      Ver : TOSVersionInfo;
   {$ENDIF}
begin
   InWinNT := False;
   HideErrorMsgs := False;

   { setup the instructions }
   If (ParamCount > 0) Then
   Begin
      InfoPanel.Visible := True;
      PanelMode := PANEL_INFO;
      TMEXPanel.Visible := False;
      ImagePanel.Visible := True;
      NextButton.Enabled := True;
      BackButton.Enabled := False;
      QuickInfoLabel.Visible := False;
      AutoDetectButton.Visible := False;
   End
   Else
   Begin
      InfoPanel.Visible := False;
      PanelMode := PANEL_PORT;
      TMEXPanel.Visible := True;
      ImagePanel.Visible := False;
      BackButton.Visible := False;
      NextButton.Caption := 'OK';
      NextButton.Enabled := True;
      QuickInfoLabel.Visible := True;
      AutoDetectButton.Visible := True;
   End;

   {$IFDEF WIN32}
      { check for WinNT }
      Ver.dwOSVersionInfoSize := SizeOf(TOSVersionInfo);
      GetVersionEx(Ver);
      If (Ver.dwPlatformId = VER_PLATFORM_WIN32_NT) Then
      Begin
        (* Removed (3.10) because 95 and NT DS1410E driver the same now
         If (Not RegWriteValue('TYPE2','IB10ENT.DLL')) Then
            Application.MessageBox('Failed to save the LPT NT setup!',
            'MicroLan Port Selection',MB_APPLMODAL Or MB_OK Or MB_ICONSTOP);
         *)

         (* Removed (3.20A) If (ParamCount > 0) Then  { (3.10) }
            Application.MessageBox('LPT (DS1410E) support is not available until after a shutdown of Windows NT!',
            '1-Wire Net Port Selection',MB_APPLMODAL Or MB_OK Or MB_ICONSTOP);
         *)
         InWinNT := True;
      End;
   {$ENDIF}

   { dynamically label the available drivers }
   cnt := 0;
   For x := 0 to 15 Do
   Begin
      If (TMGetTypeVersion(x,@buf) > 0) Then
      Begin
         DeviceTab.Tabs.Add(' ' + getToken(StrPas(buf),TOKEN_DEV) + ' ');
         PortTypeValues[cnt] := x;
         Inc(cnt);
      End;
   End;

   { check for no drivers }
   If (cnt = 0) Then
   Begin
      MessageBeep(MB_OK);
      Application.MessageBox('No iButton-TMEX drivers found, installation error!',
                              '1-Wire Net Port Selection',MB_APPLMODAL Or MB_OK Or MB_ICONSTOP);
      Application.Terminate;  { end program }
   End
   Else
   Begin
      { select the first available port type }
      GetDefaults;

      { initialized flag }
      EatEvent := False;
      GotCancel := False;
      PortName := '';
   End;
End;

{----------------------------------------------------------------------------}
{ Select the port
}
Procedure TPortForm.GetDefaults;
Var
   RetStr : Array[0..120] of Char;
   tstr   : Array[0..120] of Char;
   DefaultTab : Integer;
Begin
   { try to default the tab to DS9097U if possible }
   If (DeviceTab.Tabs.Count >= 3) Then
      DefaultTab := 2
   Else
      DefaultTab := 0;
   {Check in IBUTTON.INI file for port type and port number }
   {First try the TMEX general defaults and then iBView specifics}
   {$IFDEF WIN32}
      IBReadProfile('3.00','DefaultPortType',StrPCopy(tstr,IntToStr(PortTypeValues[DefaultTab])),
                    @RetStr,120,'IBUTTON.INI');
   {$ELSE}
      IBReadProfile('iButton TMEX General Settings','DefaultPortType',
                     StrPCopy(tstr,IntToStr(PortTypeValues[DefaultTab])),@RetStr,120,'IBUTTON.INI');
   {$ENDIF}
   Try
      TPortType := StrToInt(StrPas(RetStr));
   Except
      TPortType := PortTypeValues[0];
   End;
   {$IFDEF WIN32}
      IBReadProfile('3.00','DefaultPortNum','1',@RetStr,120,'IBUTTON.INI');
   {$ELSE}
      IBReadProfile('iButton TMEX General Settings','DefaultPortNum','1',@RetStr,120,'IBUTTON.INI');
   {$ENDIF}
   Try
      TPortNum := StrToInt(StrPas(RetStr));
   Except
      TPortNum := 1;
   End;
End;

{----------------------------------------------------------------------------}
{ Select the port
}
Function TPortForm.SelectPort(NPortNum, NPortType : SmallInt): Boolean;
Var
   tab : SmallInt;
   rt : Boolean;
   buf : array[0..200] of Char;
   infoStr,abrStr : String;
Begin
   rt := False;
   For tab := 0 to 15 Do
   Begin
      If (PortTypeValues[tab] = NPortType) Then
      Begin
         { select that tab (if not already selected) }
         If (DeviceTab.TabIndex <> tab) Then
         Begin
            { eat the change tab event }
            EatEvent := True;
            DeviceTab.TabIndex := tab;
            EatEvent := False;
         End;

         If (TMGetTypeVersion(NPortType,@buf) > 0) Then
         Begin
            infoStr := StrPas(buf);
            { get the abreviation }
            abrStr := getToken(infoStr,TOKEN_ABRV);
            { translate to a PC port if possible }
            If (Pos('COM',UpperCase(infoStr)) > 0) Then
               abrStr := 'COM'
            Else If (Pos('LPT',UpperCase(infoStr)) > 0) Then
               abrStr := 'LPT';
            PortTypeLabel.Caption := abrStr;
            { adapter type }
            AdapterLabel.Caption := getToken(infoStr,TOKEN_DEV);
            { Version }
            VersionLabel.Caption := getToken(infoStr,TOKEN_VER);
            { Date }
            DateLabel.Caption := getToken(infoStr,TOKEN_DATE);
            { misc }
            MessureLabel.Caption := getToken(infoStr,TOKEN_TAIL);
            If (MessureLabel.Width < (AdvanceGroup.ClientWidth - MiscMemo.Left)) Then
               MiscMemo.ScrollBars := ssNone
            Else
               MiscMemo.ScrollBars := ssHorizontal;
            MiscMemo.Text := MessureLabel.Caption;

            { number }
            PortNumSpin.Value := NPortNum;
            { port name }
            PortName := ' ' +  AdapterLabel.Caption + ' <' + PortTypeLabel.Caption
                + IntToStr(TPortNum) + '>';
            { success }
            rt := True;
            Break;
         End;
      End;
   End;

   SelectPort := rt;
End;

{----------------------------------------------------------------------------}
{ Parse the version string for a token
}
Function TPortForm.getToken(verStr : String; token : SmallInt): String;
Var
   currentToken : SmallInt;
   i : SmallInt;
   rtStr : String;
   inToken : Boolean;
Begin
   rtStr := '';
   currentToken := -1;
   inToken := False;
   For i := 1 to length(verStr) Do
   Begin
      If ((verStr[i] <> ' ') And (Not inToken)) Then
      Begin
         inToken := True;
         Inc(currentToken);
      End;

      If ((verStr[i] = ' ') And (inToken)) Then
         inToken := False;

      If (((inToken) And (currentToken = token)) Or
          ((token = 255) And (currentToken > TOKEN_DATE))) Then
         rtStr := rtStr + verStr[i];
   End;

   getToken := rtStr;
End;

{----------------------------------------------------------------------------}
{ Idle action for port search
}
Function TPortForm.PortCheck: Boolean;
Var
   T : DWORD;
   tbuf: Array[0..255] of Char;
   ans,flag,i,cnt: SmallInt;
   ErrorMsg: String;
Begin
   ErrorMsg := '';
   { attempt to start a session on port, timeout 10 seconds }
   T := GetTickCount;
   Repeat
      SHandle := TMExtendedStartSession(TPortNum,TPortType,nil);
      { session successfull }
      If (SHandle > 0) Then
         Break;
      { release thread to system }
      Application.ProcessMessages;
   Until ((SHandle < 0) or ((GetTickCount - T) > 10000));

   { check if could get one }
   If (SHandle > 0) Then
   Begin
      { do the setup }
      flag := TMSetup(SHandle);
      { for Mike Bolan's Laptop }
      if (flag = 0) Then
      Begin
         For i := 0 to 2 do
            flag := TMSetup(SHandle);
      End;

      { check for short condition (slow to power up brick) }
      If ((flag = 1) Or (flag = 2)) Then
      Begin
         cnt := 0;
         For i := 0 to 20 do
         Begin
            { time the setup }
            T := GetTickCount;
            ans := TMTouchReset(SHandle);
            { check for verrrry lonnnng reset }
            If ((GetTickCount - T) > 280) Then  { 3.10) }
               ans := 3;

            If ((ans < 3) And (ans >= 0)) Then
               Inc(cnt);
         End;

         If (cnt <= 4) Then
            ErrorMsg := '1-Wire Net port does not respond correctly:'
         { check for ID on parallel devices, since reset does not fail (3.10) }
         Else If (PortTypeLabel.Caption = 'LPT') Then
         Begin
            If (TMFirst(SHandle,@StateBuf[0]) <= 0) Then
               ErrorMsg := 'ID on Adapter not detected:';
         End;
      End
      Else If (flag = 3) Then
         ErrorMsg := '1-Wire Net port does not exist:'
      Else
         ErrorMsg := '1-Wire Net port does not respond correctly:';
      { do a close to release if a COM port (3.10) }
      TMClose(SHandle);
      { end the session }
      TMEndSession(SHandle);
   End
   Else If (SHandle = 0) Then
       ErrorMsg := 'Could not get access to the selected 1-Wire Net port:'
   Else
       ErrorMsg := '1-Wire Net port does not exist:';

   { check for an error }
   If (Length(ErrorMsg) > 0) Then
   Begin
      { error message }
      ErrorMsg := ErrorMsg + Chr($0A) + Chr($0D) +
                     '        '+ PortName + Chr($0A) + Chr($0D) +
                     'Check hardware and select another!';

      If (Not HideErrorMsgs) Then
      Begin
         MessageBeep(MB_ICONHAND);
         Application.MessageBox(StrPCopy(tbuf,ErrorMsg),'1-Wire Net Port Selection',
             MB_APPLMODAL Or MB_OK Or MB_ICONSTOP);
      End;
      PortCheck := False;
   End
   Else
      PortCheck := True;
End;

{----------------------------------------------------------------------------}
{ Failed to find a Port/Type so end the program
}
procedure TPortForm.CancelButtonClick(Sender: TObject);
begin
   MessageBeep(MB_ICONHAND);
   If (ParamCount > 0) Then  { (3.10) }
   Begin
      If (Application.MessageBox('Abort selecting a default 1-Wire Net and continue with the installation?',
          '1-Wire Net Port Selection',MB_APPLMODAL Or MB_OKCANCEL Or MB_ICONSTOP)
          = ID_OK) Then
          Application.Terminate;
   End
   Else
   Begin
      If (Application.MessageBox('Abort selecting a default 1-Wire Net port?',
          '1-Wire Net Port Selection',MB_APPLMODAL Or MB_OKCANCEL Or MB_ICONSTOP)
          = ID_OK) Then
          Application.Terminate;
   End;
end;

{----------------------------------------------------------------------------}
{ Check port and if successfull the save setting and quit
}
procedure TPortForm.NextButtonClick(Sender: TObject);
Var
   tbuf: Array[0..255] of Char;
   PortOK: Boolean;
begin
   If (PanelMode = PANEL_INFO) Then
   Begin
      { go to select mode }
      BackButton.Enabled := True;
      InfoPanel.Visible := False;
      AutoDetectButton.Visible := True;
      PanelMode := PANEL_PORT;
      NextButton.SetFocus;
   End
   Else
   Begin
      { get the selected port }
      TPortNum := PortNumSpin.Value;
      TPortType := PortTypeValues[DeviceTab.TabIndex];
      PortName := ' ' +  AdapterLabel.Caption + ' <' + PortTypeLabel.Caption
                + IntToStr(TPortNum) + '>';
      { Check for windows NT LPT exception }
      (* REMOVED (3.20A)
      If (InWinNT And (PortTypeLabel.Caption = 'LPT') And (ParamCount > 0)) Then  { (3.10) }
      Begin
         Application.MessageBox('The type selected cannot be tested until after a shutdown!',
               '1-Wire Net Port Selection',MB_APPLMODAL Or MB_OK Or MB_ICONSTOP);
         PortOK := True;
      End
      { really check the port }
      Else
      *)
         PortOK := PortCheck;

      { if port ok then save settings }
      If (PortOK) Then
      Begin
         If (RegWriteValue('DefaultPortNum',StrPCopy(tbuf,IntToStr(TPortNum)))) Then
            If (RegWriteValue('DefaultPortType',StrPCopy(tbuf,IntToStr(TPortType)))) Then
               Application.Terminate;

         Application.MessageBox('Failed to save the default 1-Wire Net port!',
               '1-Wire Net Port Selection',MB_APPLMODAL Or MB_OK Or MB_ICONSTOP);
         Application.Terminate;
      End;
   End;
end;

{----------------------------------------------------------------------------}
{ Replace profile write
}
Function TPortForm.RegWriteValue(KeyBuf,ValueBuf : PChar): Boolean;
Var
{$IFDEF WIN32}
   GlobKey,SoftKey,DalKey,iBViewKey,ThreeKey : HKEY;
   rtval : Boolean;
   Exhist : DWORD;
{$ENDIF}
   flag : LongInt;
Begin
{$IFDEF WIN32}
   rtval := False;

   { check for ibview specific read }
   flag := RegOpenKeyEx(HKEY_LOCAL_MACHINE,
          'Software\Dallas Semiconductor\iButton TMEX\3.00',
           0,KEY_WRITE,GlobKey);

   { could not open global hkey }
   If (flag <> ERROR_SUCCESS) Then
   Begin
      If (RegOpenKeyEx(HKEY_LOCAL_MACHINE,'Software',0,
          KEY_WRITE,SoftKey) = ERROR_SUCCESS) Then
      Begin
         If (RegCreateKeyEx(SoftKey,'Dallas Semiconductor',0,'Local Machine Data',
             REG_OPTION_NON_VOLATILE,KEY_WRITE,nil,DalKey,@Exhist)
             = ERROR_SUCCESS) Then
         Begin
            If (RegCreateKeyEx(DalKey,'iButton TMEX',0,'Local Machine Data',
                REG_OPTION_NON_VOLATILE,KEY_WRITE,nil,iBViewKey,@Exhist)
                = ERROR_SUCCESS) Then
            Begin
               If (RegCreateKeyEx(iBViewKey,'3.00',0,'Local Machine Data',
                   REG_OPTION_NON_VOLATILE,KEY_WRITE,nil,ThreeKey,@Exhist)
                   = ERROR_SUCCESS) Then
                  RegCloseKey(ThreeKey);
               RegCloseKey(iBViewKey);
            End;
            RegCloseKey(DalKey);
         End;
         RegCloseKey(SoftKey);
      End;

      { try again to open the full key }
      flag := RegOpenKeyEx(HKEY_LOCAL_MACHINE,
             'Software\Dallas Semiconductor\iButton TMEX\3.00',
              0,KEY_WRITE,GlobKey);
   End;

   { set the specific subkey value }
   If (flag = ERROR_SUCCESS) Then
   Begin
      { set the value under the key }
      If (RegSetValueEx(GlobKey,KeyBuf,0,REG_SZ,@ValueBuf[0],
          Length(ValueBuf)) = ERROR_SUCCESS) Then
          rtval := True;
      { release the key }
      RegCloseKey(GlobKey);
   End;

   { set the return value }
   RegWriteValue := rtval;
{$ELSE}
   RegWriteValue := WritePrivateProfileString('iButton TMEX General Settings',
                    KeyBuf,ValueBuf,'iButton.ini');
{$ENDIF}
End;

{----------------------------------------------------------------------------}
{ Select the default port on show
}
procedure TPortForm.FormShow(Sender: TObject);
begin
   SelectPort(TPortNum,TPortType);
   NextButton.SetFocus;
end;

{----------------------------------------------------------------------------}
{ Select a diffenent Port type
}
procedure TPortForm.DeviceTabChange(Sender: TObject; NewTab: Integer;
  var AllowChange: Boolean);
begin
   If (Not EatEvent) Then
      SelectPort(1,PortTypeValues[NewTab]);
end;

{----------------------------------------------------------------------------}
{ Go back to info panel
}
procedure TPortForm.BackButtonClick(Sender: TObject);
begin
   InfoPanel.Visible := True;
   PanelMode := PANEL_INFO;
   NextButton.Enabled := True;
   NextButton.SetFocus;
   BackButton.Enabled := False;
   AutoDetectButton.Visible := False;
end;

{----------------------------------------------------------------------------}
{ Auto detect
}
procedure TPortForm.AutoDetectButtonClick(Sender: TObject);
Var
   ans : Integer;
   tbuf: Array[0..255] of Char;
begin
   { give grave warning }
   MessageBeep(MB_ICONHAND);
   If (Application.MessageBox(
      'WARNING, Auto-Detect communicates with all available ports' +
      ' and may disturb peripheral devices. ' +  Chr($0A) + Chr($0D) +
      Chr($0A) + Chr($0D) + ' If an error occurs, ' +
      're-run Default 1-Wire Net and select port manually.',
      '1-Wire Net Port Selection',
      MB_APPLMODAL Or MB_OKCANCEL Or MB_ICONSTOP) = ID_OK) Then
   Begin
      { check for DS1410E }
      ans := ID_OK;
      (* Removed (3.20A) If (InWinNT And (PortTypeLabel.Caption = 'LPT') And (ParamCount > 0)) Then  { (3.10) }
      Begin
         { tell auto detect to skip LPT ports }
         AutoForm.SkipLPT := True;
         ans := Application.MessageBox('The DS1410E <LPT> device cannot be tested until after a shutdown'
               +  Chr($0A) + Chr($0D) + 'and will be skipped in Auto-Detect.' + Chr($0A) + Chr($0D)
               +  Chr($0A) + Chr($0D) + 'Continue with Auto-Detect anyway?',
               '1-Wire Net Port Selection',MB_APPLMODAL Or MB_OKCANCEL Or MB_ICONSTOP);
      End
      Else
      *)
         { tell auto detect to NOT skip LPT ports }
         AutoForm.SkipLPT := False;

      { check if still doing it }
      If (ans = ID_OK) Then
      Begin
         HideErrorMsgs := True;
         If (AutoForm.ShowModal = ID_OK) Then
         Begin
            If (RegWriteValue('DefaultPortNum',StrPCopy(tbuf,IntToStr(TPortNum)))) Then
               If (RegWriteValue('DefaultPortType',StrPCopy(tbuf,IntToStr(TPortType)))) Then
                  Application.Terminate;

            Application.MessageBox('Failed to save the default 1-Wire Net port!',
                  '1-Wire Net Port Selection',MB_APPLMODAL Or MB_OK Or MB_ICONSTOP);
            Application.Terminate;
         End;

         HideErrorMsgs := False;
      End;
   End;
end;

End.
