{--------------------------------------------------------------------------
 | Copyright (C) 1992,1993,1994,1995 Dallas Semiconductor Corporation.
 | All rights Reserved. Printed in U.S.A.
 | This software is protected by copyright laws of
 | the United States and of foreign countries.
 | This material may also be protected by patent laws of the United States
 | and of foreign countries.
 | This software is furnished under a license agreement and/or a
 | nondisclosure agreement and may only be used or copied in accordance
 | with the terms of those agreements.
 | The mere transfer of this software does not imply any licenses
 | of trade secrets, proprietary technology, copyrights, patents,
 | trademarks, maskwork rights, or any other form of intellectual
 | property whatsoever. Dallas Semiconductor retains all ownership rights.
 |--------------------------------------------------------------------------

   Global unit for IBVIEW.
}
unit Wtglob;

interface

Uses
   WinProcs,SysUtils;

Type
  InfoType = record
    MemType   : Byte;
    MemPages  : SmallInt;
    StatPages : Byte;
    Viewer : Array [0..2] of Byte;
    ListIndex : Byte;
  End;

Var
   IdleMode : SmallInt;
   PortMode : SmallInt;
   TypeList : Array[0..128] of Infotype;
   AppName : Array[0..50] of Char;
   PAppName : String;
   MaxType : SmallInt;
Const
   { Idle modes }
   I_GET_PORT = 1;
   I_ROM_SEARCH = 2;

   { port modes }
   P_SHOW_GET_PORT = 1;
   P_GET_PORT = 2;
   P_GET_SESSION = 3;
   P_DO_SETUP = 4;
   P_ERROR_MSG = 10;

   { memory types }
   MEM_NV = 1;
   MEM_EPROM = 2;
   MEM_EE = 3;
   MEM_MNV = 4;


   {$IFDEF WIN32}
      MaxAvail = 45000;
   {$ENDIF}

Function HexCharToByte(ch : Char): Byte;
Function ConvertToBin(instr: String; Buf : PChar): SmallInt;
Function IBReadProfile(ProgBuf,KeyBuf,DefBuf,RetBuf : PChar;
           RetLen : SmallInt; IniBuf : Pchar): SmallInt;
Function IBWriteProfile(ProgBuf,KeyBuf,ValueBuf,IniBuf : PChar): Boolean;

implementation


{----------------------------------------------------------------------------}
{ Replace profile read
}
Function IBReadProfile(ProgBuf,KeyBuf,DefBuf,RetBuf : PChar;
           RetLen : SmallInt; IniBuf : Pchar): SmallInt;
Var
{$IFDEF WIN32}
   GlobKey,SubKey : HKEY;
   rtval : SmallInt;
   RtType,RtSize : DWORD;
{$ENDIF}
   flag : LongInt;
Begin
{$IFDEF WIN32}
   rtval := -1; flag := -1;

   { check for ibview specific read }
   If (StrIComp(IniBuf,'IBVIEW.INI') = 0) Then
   Begin
      { get a hkey to the global area }
      flag := RegOpenKeyEx(HKEY_CURRENT_USER,
          'Software\Dallas Semiconductor\iButton Viewer32\3.00',
           0,KEY_READ,GlobKey);
   End
   { check for a iButton TMEX specific read }
   Else If (StrIComp(IniBuf,'IBUTTON.INI') = 0) Then
   Begin
      flag := RegOpenKeyEx(HKEY_LOCAL_MACHINE,
          'Software\Dallas Semiconductor\iButton TMEX',
           0,KEY_READ,GlobKey);
   End;

   { get the specific subkey value }
   If (flag = ERROR_SUCCESS) Then
   Begin
      { get a hkey to the specific sub-key }
      If (RegOpenKeyEx(GlobKey,ProgBuf,
          0,KEY_READ,SubKey) = ERROR_SUCCESS) Then
      Begin
         { read the value under the key }
         RtSize := RetLen;
         If (RegQueryValueEx(SubKey,KeyBuf,nil,@RtType,
             @RetBuf[0],@RtSize) = ERROR_SUCCESS) Then
         Begin
            { check to verify that the return type is string }
            If (RtType = REG_SZ) Then
            Begin
               rtval := RtSize; { success! }
               RetBuf[RtSize] := Chr(0);   { zero terminate string }
            End
         End;
         { release the key }
         RegCloseKey(SubKey);
      End;
      { release the key }
      RegCloseKey(GlobKey);
   End;

   { check for failure to find the key }
   If (rtval = -1) Then
   Begin
      { so copy the default to the return buffer }
      If (StrLen(DefBuf) <= Cardinal(RetLen)) Then
      Begin
         rtval := StrLen(DefBuf);
         StrCopy(RetBuf,DefBuf);
      End
      Else
      Begin
         StrLCopy(RetBuf,DefBuf,RetLen-2);
         rtval := RetLen-2;
      End;
   End;
   { set the return value }
   IBReadProfile := rtval;
{$ELSE}
   IBReadProfile := GetPrivateProfileString(ProgBuf,KeyBuf,DefBuf,RetBuf,RetLen,IniBuf);
{$ENDIF}
End;

{----------------------------------------------------------------------------}
{ Replace profile write
}
Function IBWriteProfile(ProgBuf,KeyBuf,ValueBuf,IniBuf : PChar): Boolean;
Var
{$IFDEF WIN32}
   GlobKey,SubKey,SoftKey,DalKey,iBViewKey,ThreeKey : HKEY;
   rtval : Boolean;
   Exhist : DWORD;
{$ENDIF}
   flag : LongInt;
Begin
{$IFDEF WIN32}
   rtval := False;

   { check for ibview specific read }
   flag := RegOpenKeyEx(HKEY_CURRENT_USER,
          'Software\Dallas Semiconductor\iButton Viewer32\3.00',
           0,KEY_WRITE,GlobKey);

   { could not open global hkey }
   If (flag <> ERROR_SUCCESS) Then
   Begin
      If (RegOpenKeyEx(HKEY_CURRENT_USER,'Software',0,
          KEY_WRITE,SoftKey) = ERROR_SUCCESS) Then
      Begin
         If (RegCreateKeyEx(SoftKey,'Dallas Semiconductor',0,'Per-User Data',
             REG_OPTION_NON_VOLATILE,KEY_WRITE,nil,DalKey,@Exhist)
             = ERROR_SUCCESS) Then
         Begin
            If (RegCreateKeyEx(DalKey,'iButton Viewer32',0,'Per-User Data',
                REG_OPTION_NON_VOLATILE,KEY_WRITE,nil,iBViewKey,@Exhist)
                = ERROR_SUCCESS) Then
            Begin
               If (RegCreateKeyEx(iBViewKey,'3.00',0,'Per-User Data',
                   REG_OPTION_NON_VOLATILE,KEY_WRITE,nil,ThreeKey,@Exhist)
                   = ERROR_SUCCESS) Then
                  RegCloseKey(ThreeKey);
               RegCloseKey(iBViewKey);
            End;
            RegCloseKey(DalKey);
         End;
         RegCloseKey(SoftKey);
      End;

      { try again to open the full key }
      flag := RegOpenKeyEx(HKEY_CURRENT_USER,
             'Software\Dallas Semiconductor\iButton Viewer32\3.00',
              0,KEY_WRITE,GlobKey);
   End;

   { set the specific subkey value }
   If (flag = ERROR_SUCCESS) Then
   Begin
      { get a hkey to the specific sub-key }
      If (RegCreateKeyEx(GlobKey,ProgBuf,0,'Per-User Data',
          REG_OPTION_NON_VOLATILE,KEY_WRITE,nil,SubKey,@Exhist)
          = ERROR_SUCCESS) Then
      Begin
         { set the value under the key }
         If (RegSetValueEx(SubKey,KeyBuf,0,REG_SZ,@ValueBuf[0],
             Length(ValueBuf)) = ERROR_SUCCESS) Then
            rtval := True;
         { release the key }
         RegCloseKey(SubKey);
      End;
      { release the key }
      RegCloseKey(GlobKey);
   End;

   { set the return value }
   IBWriteProfile := rtval;
{$ELSE}
   IBWriteProfile := WritePrivateProfileString(ProgBuf,KeyBuf,ValueBuf,IniBuf);
{$ENDIF}
End;

{----------------------------------------------------------------------------}
{ Take a normal string and parce it into the a byte buffer.  Return the
  Length or -1 for an error.  The Buf must be at least 128 bytes long.
}
Function ConvertToBin(instr: String; Buf : PChar): SmallInt;
Var
   cnt,ln,i,rcnt,addchar : SmallInt;
   By : Byte;
   lastchar : Char;
Begin
   ln := Length(instr);
   rcnt := 0;  { string run count }
   cnt := 0;   { position count in the buffer }
   addchar := 0;
   lastchar := ' ';

   { loop to convert the chars in the string }
   For i := 1 to ln Do
   Begin
      By := HexCharToByte(instr[i]);

      { check if not to a space yet }
      If (instr[i] <> ' ') Then
      Begin
         { check if this is a non-hex }
         If (By = $FF) Then
         Begin
            If (rcnt > 0) Then
            Begin
               cnt := -1;
               Break;
            End;
         End
         Else
         Begin
            addchar := addchar shl 4;
            addchar := addchar Or By;
            If (rcnt > 1) Then
            Begin
               cnt := -1;
               Break;
            End;
         End;
         { count this char }
         lastchar := instr[i];
         Inc(rcnt);
      End
      { add it to the buffer }
      Else If (rcnt > 0) Then
      Begin
         If (rcnt = 2) Then
            Buf[cnt] := Chr(addchar)
         Else
            Buf[cnt] := lastchar;
         Inc(cnt);
         rcnt := 0;
         addchar := 0;
      End;
   End;

   { check for final run without trailing space }
   If ((cnt <> -1) And (rcnt > 0)) Then
   Begin
      If (rcnt = 2) Then
         Buf[cnt] := Chr(addchar)
      Else
         Buf[cnt] := lastchar;
      Inc(cnt);
   End;

   { return the length or -1 }
   ConvertToBin := cnt;
End;

{----------------------------------------------------------------------------}
{ Take a hex character and return the binary represetation
}
Function HexCharToByte(ch : Char): Byte;
Begin
   If ((Ord(ch) >= Ord('0')) And (Ord(ch) <= Ord('9'))) Then
      HexCharToByte := Byte(Ord(ch) - $30)
   Else If ((Ord(ch) >= Ord('A')) And (Ord(ch) <= Ord('F'))) Then
      HexCharToByte := Byte(Ord(ch) - $37)
   Else If ((Ord(ch) >= Ord('a')) And (Ord(ch) <= Ord('f'))) Then
      HexCharToByte := Byte(Ord(ch) - $57)
   Else
      HexCharToByte := $FF;
End;


end.
